@@ -12,31 +12,31 @@ Primary interfaces for the salt-cloud system
 # the VM data will be in opts['profiles']
 
 # Import python libs
-from __future__ import print_function
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import os
 import sys
 import logging
 from salt.ext.six.moves import input
 
 # Import salt libs
+import salt.cloud
 import salt.config
 import salt.defaults.exitcodes
 import salt.output
-import salt.utils
-from salt.utils import parsers
-from salt.utils.verify import check_user, verify_env, verify_files, verify_log
-
-# Import salt.cloud libs
-import salt.cloud
-import salt.utils.cloud
-from salt.exceptions import SaltCloudException, SaltCloudSystemExit
-import salt.ext.six as six
 import salt.syspaths as syspaths
+import salt.utils.cloud
+import salt.utils.parsers
+import salt.utils.user
+from salt.exceptions import SaltCloudException, SaltCloudSystemExit
+from salt.utils.verify import check_user, verify_env, verify_log_files, verify_log
+
+# Import 3rd-party libs
+from salt.ext import six
+
 log = logging.getLogger(__name__)
 
 
-class SaltCloud(parsers.SaltCloudParser):
+class SaltCloud(salt.utils.parsers.SaltCloudParser):
 
     def run(self):
         '''
@@ -47,7 +47,7 @@ class SaltCloud(parsers.SaltCloudParser):
 
         salt_master_user = self.config.get('user')
         if salt_master_user is None:
-            salt_master_user = salt.utils.get_user()
+            salt_master_user = salt.utils.user.get_user()
 
         if not check_user(salt_master_user):
             self.error(
@@ -66,16 +66,15 @@ class SaltCloud(parsers.SaltCloudParser):
             if self.config['verify_env']:
                 verify_env(
                     [os.path.dirname(self.config['conf_file'])],
-                    salt_master_user
+                    salt_master_user,
+                    root_dir=self.config['root_dir'],
                 )
                 logfile = self.config['log_file']
-                if logfile is not None and not logfile.startswith('tcp://') \
-                        and not logfile.startswith('udp://') \
-                        and not logfile.startswith('file://'):
+                if logfile is not None:
                     # Logfile is not using Syslog, verify
-                    verify_files([logfile], salt_master_user)
+                    verify_log_files([logfile], salt_master_user)
         except (IOError, OSError) as err:
-            log.error('Error while verifying the environment: {0}'.format(err))
+            log.error('Error while verifying the environment: %s', err)
             sys.exit(err.errno)
 
         # Setup log file logging
@@ -129,9 +128,7 @@ class SaltCloud(parsers.SaltCloudParser):
                     self.handle_exception(msg, exc)
 
             elif self.config.get('map', None):
-                log.info(
-                    'Applying map from \'{0}\'.'.format(self.config['map'])
-                )
+                log.info('Applying map from \'%s\'.', self.config['map'])
                 try:
                     ret = mapper.interpolated_map(
                         query=self.selected_query_option
@@ -187,7 +184,7 @@ class SaltCloud(parsers.SaltCloudParser):
                           'instances.'.format(map_file, names)
                     self.handle_exception(msg, SaltCloudSystemExit)
 
-                log.info('Applying map from \'{0}\'.'.format(map_file))
+                log.info('Applying map from \'%s\'.', map_file)
                 matching = mapper.delete_map(query='list_nodes')
             else:
                 matching = mapper.get_running_by_names(
@@ -218,9 +215,7 @@ class SaltCloud(parsers.SaltCloudParser):
         elif self.options.action and (self.config.get('names', None) or
                                       self.config.get('map', None)):
             if self.config.get('map', None):
-                log.info(
-                    'Applying map from \'{0}\'.'.format(self.config['map'])
-                )
+                log.info('Applying map from \'%s\'.', self.config['map'])
                 try:
                     names = mapper.get_vmnames_by_action(self.options.action)
                 except SaltCloudException as exc:
@@ -295,16 +290,14 @@ class SaltCloud(parsers.SaltCloudParser):
             salt.utils.cloud.store_password_in_keyring(provider_name, username)
         elif self.config.get('map', None) and \
                 self.selected_query_option is None:
-            if len(mapper.rendered_map) == 0:
+            if not mapper.rendered_map:
                 sys.stderr.write('No nodes defined in this map')
                 self.exit(salt.defaults.exitcodes.EX_GENERIC)
             try:
                 ret = {}
                 run_map = True
 
-                log.info(
-                    'Applying map from \'{0}\'.'.format(self.config['map'])
-                )
+                log.info('Applying map from \'%s\'.', self.config['map'])
                 dmap = mapper.map_data()
 
                 msg = ''
@@ -421,21 +414,19 @@ class SaltCloud(parsers.SaltCloudParser):
                     msg = 'Error: {0}'.format(msg)
                 self.exit(
                     exc.exit_code,
-                    '{0}\n'.format(
-                        msg.format(str(exc).rstrip())
-                    )
+                    msg.format(exc).rstrip() + '\n'
                 )
             # It's not a system exit but it's an error we can
             # handle
-            self.error(
-                msg.format(str(exc))
-            )
+            self.error(msg.format(exc))
         # This is a generic exception, log it, include traceback if
         # debug logging is enabled and exit.
+        # pylint: disable=str-format-in-logging
         log.error(
             msg.format(exc),
             # Show the traceback if the debug logging level is
             # enabled
             exc_info_on_loglevel=logging.DEBUG
         )
+        # pylint: enable=str-format-in-logging
         self.exit(salt.defaults.exitcodes.EX_GENERIC)
