@@ -310,6 +310,18 @@ def raw_mod(opts, name, functions, mod='modules'):
     return dict(loader._dict)  # return a copy of *just* the funcs for `name`
 
 
+def metaproxy(opts):
+    '''
+    Return functions used in the meta proxy
+    '''
+
+    return LazyLoader(
+        _module_dirs(opts, 'metaproxy'),
+        opts,
+        tag='metaproxy'
+    )
+
+
 def matchers(opts):
     '''
     Return the matcher services plugins
@@ -1706,7 +1718,7 @@ class LazyLoader(salt.utils.lazy.LazyDict):
 
         # enforce depends
         try:
-            Depends.enforce_dependencies(self._dict, self.tag)
+            Depends.enforce_dependencies(self._dict, self.tag, name)
         except RuntimeError as exc:
             log.info(
                 'Depends.enforce_dependencies() failed for the following '
@@ -1725,7 +1737,7 @@ class LazyLoader(salt.utils.lazy.LazyDict):
         if not isinstance(key, six.string_types):
             raise KeyError('The key must be a string.')
         if '.' not in key:
-            raise KeyError('The key \'%s\' should contain a \'.\'', key)
+            raise KeyError('The key \'{0}\' should contain a \'.\''.format(key))
         mod_name, _ = key.split('.', 1)
         with self._lock:
             # It is possible that the key is in the dictionary after
