@@ -11,15 +11,17 @@ import salt.utils.data  # pylint: disable=3rd-party-module-not-gated
 log = logging.getLogger(__name__)
 
 
-def match(tgt, delimiter=DEFAULT_TARGET_DELIM):
+def match(tgt, delimiter=DEFAULT_TARGET_DELIM, opts=None):
     '''
     Reads in the pillar glob match
     '''
+    if not opts:
+        opts = __opts__
     log.debug('pillar target: %s', tgt)
     if delimiter not in tgt:
         log.error('Got insufficient arguments for pillar match '
                   'statement from master')
         return False
     return salt.utils.data.subdict_match(
-        __opts__['pillar'], tgt, delimiter=delimiter
+        opts['pillar'], tgt, delimiter=delimiter
     )
