@@ -8,20 +8,20 @@ Support for Config Server Firewall (CSF)
 '''
 
 # Import Python Libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import re
 
 # Import Salt Libs
+import salt.utils.path
 from salt.exceptions import CommandExecutionError, SaltInvocationError
-import salt.utils
-from salt.ext.six.moves import map
+from salt.ext import six
 
 
 def __virtual__():
     '''
     Only load if csf exists on the system
     '''
-    if salt.utils.which('csf') is None:
+    if salt.utils.path.which('csf') is None:
         return (False,
                 'The csf execution module cannot be loaded: csf unavailable.')
     else:
@@ -77,7 +77,7 @@ def __csf_cmd(cmd):
     '''
     Execute csf command
     '''
-    csf_cmd = '{0} {1}'.format(salt.utils.which('csf'), cmd)
+    csf_cmd = '{0} {1}'.format(salt.utils.path.which('csf'), cmd)
     out = __salt__['cmd.run_all'](csf_cmd)
 
     if out['retcode'] != 0:
@@ -260,7 +260,7 @@ def skip_nics(nics, ipv6=False):
         ipv6 = '6'
     else:
         ipv6 = ''
-    nics_csv = ','.join(map(str, nics))
+    nics_csv = ','.join(six.moves.map(six.text_type, nics))
     result = __salt__['file.replace']('/etc/csf/csf.conf',
             pattern='^ETH{0}_DEVICE_SKIP(\ +)?\=(\ +)?".*"'.format(ipv6),  # pylint: disable=W1401
                                         repl='ETH{0}_DEVICE_SKIP = "{1}"'.format(ipv6, nics_csv))
@@ -552,7 +552,7 @@ def allow_ports(ports, proto='tcp', direction='in'):
     proto = proto.upper()
     direction = direction.upper()
     _validate_direction_and_proto(direction, proto)
-    ports_csv = ','.join(map(str, ports))
+    ports_csv = ','.join(six.moves.map(six.text_type, ports))
     directions = build_directions(direction)
 
     for direction in directions:
