@@ -43,13 +43,13 @@ In order to use the module if you have no docker-compose file on the server you
 can issue the command create, it takes two arguments the path where the
 docker-compose.yml will be stored and the content of this latter:
 
-.. code-block:: bash
+.. code-block:: text
 
     # salt-call -l debug dockercompose.create /tmp/toto '
-     database:
-     image: mongo:3.0
-     command: mongod --smallfiles --quiet --logpath=/dev/null
-     '
+    database:
+    image: mongo:3.0
+    command: mongod --smallfiles --quiet --logpath=/dev/null
+    '
 
 Then you can execute a list of method defined at the bottom with at least one
 argument (the path where the docker-compose.yml will be read) and an optional
@@ -95,20 +95,33 @@ Functions
     - :py:func:`dockercompose.build <salt.modules.dockercompose.build>`
 - Gather information about containers:
     - :py:func:`dockercompose.ps <salt.modules.dockercompose.ps>`
+- Manage service definitions:
+    - :py:func:`dockercompose.service_create <salt.modules.dockercompose.ps>`
+    - :py:func:`dockercompose.service_upsert <salt.modules.dockercompose.ps>`
+    - :py:func:`dockercompose.service_remove <salt.modules.dockercompose.ps>`
+    - :py:func:`dockercompose.service_set_tag <salt.modules.dockercompose.ps>`
 
 Detailed Function Documentation
 -------------------------------
 '''
 
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 import inspect
 import logging
 import os
 import re
-import salt.utils
+
+import salt.utils.files
+import salt.utils.stringutils
+
+from salt.ext import six
 
 from operator import attrgetter
+
+from salt.serializers import json
+from salt.utils import yaml
+
 try:
     import compose
     from compose.cli.command import get_project
@@ -130,12 +143,12 @@ log = logging.getLogger(__name__)
 debug = False
 
 __virtualname__ = 'dockercompose'
-dc_filename = 'docker-compose.yml'
+DEFAULT_DC_FILENAMES = ('docker-compose.yml', 'docker-compose.yaml')
 
 
 def __virtual__():
     if HAS_DOCKERCOMPOSE:
-        match = re.match(VERSION_RE, str(compose.__version__))
+        match = re.match(VERSION_RE, six.text_type(compose.__version__))
         if match:
             version = tuple([int(x) for x in match.group(1).split('.')])
             if version >= MIN_DOCKERCOMPOSE:
@@ -168,33 +181,108 @@ def __standardize_result(status, message, data=None, debug_msg=None):
     return result
 
 
-def __read_docker_compose(path):
+def __get_docker_file_path(path):
     '''
-    Read the docker-compose.yml file if it exists in the directory
+    Determines the filepath to use
 
     :param path:
     :return:
     '''
-    if os.path.isfile(os.path.join(path, dc_filename)) is False:
+    if os.path.isfile(path):
+        return path
+    for dc_filename in DEFAULT_DC_FILENAMES:
+        file_path = os.path.join(path, dc_filename)
+        if os.path.isfile(file_path):
+            return file_path
+    # implicitly return None
+
+
+def __read_docker_compose_file(file_path):
+    '''
+    Read the compose file if it exists in the directory
+
+    :param file_path:
+    :return:
+    '''
+    if not os.path.isfile(file_path):
         return __standardize_result(False,
-                                    'Path does not exist or docker-compose.yml is not present',
+                                    'Path {} is not present'.format(file_path),
                                     None, None)
-    f = salt.utils.fopen(os.path.join(path, dc_filename), 'r')  # pylint: disable=resource-leakage
-    result = {'docker-compose.yml': ''}
-    if f:
-        for line in f:
-            result['docker-compose.yml'] += line
-        f.close()
-    else:
-        return __standardize_result(False, 'Could not read docker-compose.yml file.',
+    try:
+        with salt.utils.files.fopen(file_path, 'r') as fl:
+            file_name = os.path.basename(file_path)
+            result = {file_name: ''}
+            for line in fl:
+                result[file_name] += salt.utils.stringutils.to_unicode(line)
+    except EnvironmentError:
+        return __standardize_result(False,
+                                    'Could not read {0}'.format(file_path),
                                     None, None)
-    return __standardize_result(True, 'Reading content of docker-compose.yml file',
+    return __standardize_result(True,
+                                'Reading content of {0}'.format(file_path),
                                 result, None)
 
 
-def __write_docker_compose(path, docker_compose):
+def __load_docker_compose(path):
     '''
-    Write docker-compose to a temp directory
+    Read the compose file and load its' contents
+
+    :param path:
+    :return:
+    '''
+    file_path = __get_docker_file_path(path)
+    if file_path is None:
+        msg = 'Could not find docker-compose file at {0}'.format(path)
+        return None, __standardize_result(False, msg,
+                                          None, None)
+    if not os.path.isfile(file_path):
+        return None, __standardize_result(False,
+                                    'Path {} is not present'.format(file_path),
+                                    None, None)
+    try:
+        with salt.utils.files.fopen(file_path, 'r') as fl:
+            loaded = yaml.load(fl)
+    except EnvironmentError:
+        return None, __standardize_result(False,
+                                    'Could not read {0}'.format(file_path),
+                                    None, None)
+    except yaml.YAMLError as yerr:
+        msg = 'Could not parse {0} {1}'.format(file_path, yerr)
+        return None, __standardize_result(False, msg,
+                                    None, None)
+    if not loaded:
+        msg = 'Got empty compose file at {0}'.format(file_path)
+        return None, __standardize_result(False, msg,
+                                          None, None)
+    if 'services' not in loaded:
+        loaded['services'] = {}
+    result = {
+         'compose_content': loaded,
+         'file_name': os.path.basename(file_path)
+    }
+    return result, None
+
+
+def __dump_docker_compose(path, content, already_existed):
+    '''
+    Dumps
+
+    :param path:
+    :param content: the not-yet dumped content
+    :return:
+    '''
+    try:
+        dumped = yaml.safe_dump(content, indent=2, default_flow_style=False)
+        return __write_docker_compose(path, dumped, already_existed)
+    except TypeError as t_err:
+        msg = 'Could not dump {0} {1}'.format(content, t_err)
+        return __standardize_result(False, msg,
+                                    None, None)
+
+
+def __write_docker_compose(path, docker_compose, already_existed):
+    '''
+    Write docker-compose to a path
     in order to use it with docker-compose ( config check )
 
     :param path:
@@ -204,38 +292,111 @@ def __write_docker_compose(path, docker_compose):
 
     :return:
     '''
-
-    if os.path.isdir(path) is False:
-        os.mkdir(path)
-    f = salt.utils.fopen(os.path.join(path, dc_filename), 'w')  # pylint: disable=resource-leakage
-    if f:
-        f.write(docker_compose)
-        f.close()
+    if path.lower().endswith(('.yml', '.yaml')):
+        file_path = path
+        dir_name = os.path.dirname(path)
     else:
+        dir_name = path
+        file_path = os.path.join(dir_name, DEFAULT_DC_FILENAMES[0])
+    if os.path.isdir(dir_name) is False:
+        os.mkdir(dir_name)
+    try:
+        with salt.utils.files.fopen(file_path, 'w') as fl:
+            fl.write(salt.utils.stringutils.to_str(docker_compose))
+    except EnvironmentError:
         return __standardize_result(False,
-                                    'Could not write docker-compose file in {0}'.format(path),
+                                    'Could not write {0}'.format(file_path),
                                     None, None)
-    project = __load_project(path)
+    project = __load_project_from_file_path(file_path)
     if isinstance(project, dict):
-        os.remove(os.path.join(path, dc_filename))
+        if not already_existed:
+            os.remove(file_path)
         return project
-    return path
+    return file_path
 
 
 def __load_project(path):
     '''
     Load a docker-compose project from path
 
+    :param path:
+    :return:
+    '''
+    file_path = __get_docker_file_path(path)
+    if file_path is None:
+        msg = 'Could not find docker-compose file at {0}'.format(path)
+        return __standardize_result(False,
+                                    msg,
+                                    None, None)
+    return __load_project_from_file_path(file_path)
+
+
+def __load_project_from_file_path(file_path):
+    '''
+    Load a docker-compose project from file path
+
     :param path:
     :return:
     '''
     try:
-        project = get_project(path)
+        project = get_project(project_dir=os.path.dirname(file_path),
+                              config_path=[os.path.basename(file_path)])
     except Exception as inst:
         return __handle_except(inst)
     return project
 
 
+def __load_compose_definitions(path, definition):
+    '''
+    Will load the compose file located at path
+    Then determines the format/contents of the sent definition
+
+    err or results are only set if there were any
+
+    :param path:
+    :param definition:
+    :return tuple(compose_result, loaded_definition, err):
+    '''
+    compose_result, err = __load_docker_compose(path)
+    if err:
+        return None, None, err
+    if isinstance(definition, dict):
+        return compose_result, definition, None
+    elif definition.strip().startswith('{'):
+        try:
+            loaded_definition = json.deserialize(definition)
+        except json.DeserializationError as jerr:
+            msg = 'Could not parse {0} {1}'.format(definition, jerr)
+            return None, None, __standardize_result(False, msg,
+                                              None, None)
+    else:
+        try:
+            loaded_definition = yaml.load(definition)
+        except yaml.YAMLError as yerr:
+            msg = 'Could not parse {0} {1}'.format(definition, yerr)
+            return None, None, __standardize_result(False, msg,
+                                              None, None)
+    return compose_result, loaded_definition, None
+
+
+def __dump_compose_file(path, compose_result, success_msg, already_existed):
+    '''
+    Utility function to dump the compose result to a file.
+
+    :param path:
+    :param compose_result:
+    :param success_msg: the message to give upon success
+    :return:
+    '''
+    ret = __dump_docker_compose(path,
+                                compose_result['compose_content'],
+                                already_existed)
+    if isinstance(ret, dict):
+        return ret
+    return __standardize_result(True, success_msg,
+                                compose_result['compose_content'], None)
+
+
 def __handle_except(inst):
     '''
     Handle exception and return a standard result
@@ -286,8 +447,12 @@ def get(path):
 
         salt myminion dockercompose.get /path/where/docker-compose/stored
     '''
-
-    salt_result = __read_docker_compose(path)
+    file_path = __get_docker_file_path(path)
+    if file_path is None:
+        return __standardize_result(False,
+                                    'Path {} is not present'.format(path),
+                                    None, None)
+    salt_result = __read_docker_compose_file(file_path)
     if not salt_result['status']:
         return salt_result
     project = __load_project(path)
@@ -315,14 +480,19 @@ def create(path, docker_compose):
         salt myminion dockercompose.create /path/where/docker-compose/stored content
     '''
     if docker_compose:
-        ret = __write_docker_compose(path, docker_compose)
+        ret = __write_docker_compose(path,
+                                     docker_compose,
+                                     already_existed=False)
         if isinstance(ret, dict):
             return ret
     else:
         return __standardize_result(False,
                                     'Creating a docker-compose project failed, you must send a valid docker-compose file',
                                     None, None)
-    return __standardize_result(True, 'Successfully created the docker-compose file', {'compose.base_dir': path}, None)
+    return __standardize_result(True,
+                                'Successfully created the docker-compose file',
+                                {'compose.base_dir': path},
+                                None)
 
 
 def pull(path, service_names=None):
@@ -686,7 +856,7 @@ def ps(path):
 
 def up(path, service_names=None):
     '''
-    Create and start containers defined in the the docker-compose.yml file
+    Create and start containers defined in the docker-compose.yml file
     located in path, service_names is a python list, if omitted create and
     start all containers
 
@@ -719,3 +889,137 @@ def up(path, service_names=None):
         except Exception as inst:
             return __handle_except(inst)
     return __standardize_result(True, 'Adding containers via docker-compose', result, debug_ret)
+
+
+def service_create(path, service_name, definition):
+    '''
+    Create the definition of a docker-compose service
+    This fails when the service already exists
+    This does not pull or up the service
+    This wil re-write your yaml file. Comments will be lost. Indentation is set to 2 spaces
+
+    path
+        Path where the docker-compose file is stored on the server
+    service_name
+        Name of the service to create
+    definition
+        Service definition as yaml or json string
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt myminion dockercompose.service_create /path/where/docker-compose/stored service_name definition
+    '''
+    compose_result, loaded_definition, err = __load_compose_definitions(path, definition)
+    if err:
+        return err
+    services = compose_result['compose_content']['services']
+    if service_name in services:
+        msg = 'Service {0} already exists'.format(service_name)
+        return __standardize_result(False, msg, None, None)
+    services[service_name] = loaded_definition
+    return __dump_compose_file(path, compose_result,
+                               'Service {0} created'.format(service_name),
+                               already_existed=True)
+
+
+def service_upsert(path, service_name, definition):
+    '''
+    Create or update the definition of a docker-compose service
+    This does not pull or up the service
+    This wil re-write your yaml file. Comments will be lost. Indentation is set to 2 spaces
+
+    path
+        Path where the docker-compose file is stored on the server
+    service_name
+        Name of the service to create
+    definition
+        Service definition as yaml or json string
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt myminion dockercompose.service_upsert /path/where/docker-compose/stored service_name definition
+    '''
+    compose_result, loaded_definition, err = __load_compose_definitions(path, definition)
+    if err:
+        return err
+    services = compose_result['compose_content']['services']
+    if service_name in services:
+        msg = 'Service {0} already exists'.format(service_name)
+        return __standardize_result(False, msg, None, None)
+    services[service_name] = loaded_definition
+    return __dump_compose_file(path, compose_result,
+                               'Service definition for {0} is set'.format(service_name),
+                               already_existed=True)
+
+
+def service_remove(path, service_name):
+    '''
+    Remove the definition of a docker-compose service
+    This does not rm the container
+    This wil re-write your yaml file. Comments will be lost. Indentation is set to 2 spaces
+
+    path
+        Path where the docker-compose file is stored on the server
+    service_name
+        Name of the service to remove
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt myminion dockercompose.service_remove /path/where/docker-compose/stored service_name
+    '''
+    compose_result, err = __load_docker_compose(path)
+    if err:
+        return err
+    services = compose_result['compose_content']['services']
+    if service_name not in services:
+        return __standardize_result(False,
+                                    'Service {0} did not exists'.format(service_name),
+                                    None, None)
+    del services[service_name]
+    return __dump_compose_file(path, compose_result,
+                               'Service {0} is removed from {1}'.format(service_name, path),
+                               already_existed=True)
+
+
+def service_set_tag(path, service_name, tag):
+    '''
+    Change the tag of a docker-compose service
+    This does not pull or up the service
+    This wil re-write your yaml file. Comments will be lost. Indentation is set to 2 spaces
+
+    path
+        Path where the docker-compose file is stored on the server
+    service_name
+        Name of the service to remove
+    tag
+        Name of the tag (often used as version) that the service image should have
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt myminion dockercompose.service_create /path/where/docker-compose/stored service_name tag
+    '''
+    compose_result, err = __load_docker_compose(path)
+    if err:
+        return err
+    services = compose_result['compose_content']['services']
+    if service_name not in services:
+        return __standardize_result(False,
+                                    'Service {0} did not exists'.format(service_name),
+                                    None, None)
+    if 'image' not in services[service_name]:
+        return __standardize_result(False,
+                                    'Service {0} did not contain the variable "image"'.format(service_name),
+                                    None, None)
+    image = services[service_name]['image'].split(':')[0]
+    services[service_name]['image'] = '{0}:{1}'.format(image, tag)
+    return __dump_compose_file(path, compose_result,
+                               'Service {0} is set to tag "{1}"'.format(service_name, tag),
+                               already_existed=True)
