@@ -24,15 +24,19 @@ For example:
       # optional: some functions require a repo_name, which
       # can be set in the config file, or passed in at the CLI.
       repo_name: my_repo
+
+      # optional: it can be dangerous to change the privacy of a repository
+      # in an automated way. set this to True to allow privacy modifications
+      allow_repo_privacy_changes: False
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 import logging
 
 # Import Salt Libs
 from salt.exceptions import CommandExecutionError
-import salt.ext.six as six
+from salt.ext import six
 import salt.utils.http
 
 # Import third party libs
@@ -81,7 +85,7 @@ def _get_config_value(profile, config_name):
         )
 
     config_value = config.get(config_name)
-    if not config_value:
+    if config_value is None:
         raise CommandExecutionError(
             'The \'{0}\' parameter was not found in the \'{1}\' '
             'profile.'.format(
@@ -106,6 +110,7 @@ def _get_client(profile):
     if key not in __context__:
         __context__[key] = github.Github(
             token,
+            per_page=100
         )
     return __context__[key]
 
@@ -121,9 +126,8 @@ def _get_members(organization, params=None):
 
 def _get_repos(profile, params=None, ignore_cache=False):
     # Use cache when no params are given
-    key = 'github.{0}:repos'.format(
-        _get_config_value(profile, 'org_name')
-    )
+    org_name = _get_config_value(profile, 'org_name')
+    key = 'github.{0}:repos'.format(org_name)
 
     if key not in __context__ or ignore_cache or params is not None:
         org_name = _get_config_value(profile, 'org_name')
@@ -136,9 +140,22 @@ def _get_repos(profile, params=None, ignore_cache=False):
             organization.url + '/repos',
             params
         )
+
+        # Only cache results if no params were given (full scan)
         if params is not None:
             return result
-        __context__[key] = list(result)  # force evaluation of all pages
+
+        next_result = []
+
+        for repo in result:
+            next_result.append(repo)
+
+            # Cache a copy of each repo for single lookups
+            repo_key = "github.{0}:{1}:repo_info".format(org_name, repo.name.lower())
+            __context__[repo_key] = _repo_to_dict(repo)
+
+        __context__[key] = next_result
+
     return __context__[key]
 
 
@@ -209,8 +226,8 @@ def get_user(name, profile='github', user_details=False):
 
     try:
         user = client.get_user(name)
-    except UnknownObjectException as e:
-        log.exception("Resource not found {0}: ".format(str(e)))
+    except UnknownObjectException:
+        log.exception("Resource not found")
         return False
 
     response['company'] = user.company
@@ -228,7 +245,7 @@ def get_user(name, profile='github', user_details=False):
             "GET",
             organization.url + "/memberships/" + user._identity
         )
-    except UnknownObjectException as e:
+    except UnknownObjectException:
         response['membership_state'] = 'nonexistent'
         response['in_org'] = False
         return response
@@ -263,8 +280,8 @@ def add_user(name, profile='github'):
 
     try:
         github_named_user = client.get_user(name)
-    except UnknownObjectException as e:
-        log.exception("Resource not found {0}: ".format(str(e)))
+    except UnknownObjectException:
+        log.exception("Resource not found")
         return False
 
     headers, data = organization._requester.requestJsonAndCheck(
@@ -299,8 +316,8 @@ def remove_user(name, profile='github'):
 
     try:
         git_user = client.get_user(name)
-    except UnknownObjectException as e:
-        log.exception("Resource not found: {0}".format(str(e)))
+    except UnknownObjectException:
+        log.exception("Resource not found")
         return False
 
     if organization.has_in_members(git_user):
@@ -343,7 +360,7 @@ def get_issue(issue_number, repo_name=None, profile='github', output='min'):
         repo_name = _get_config_value(profile, 'repo_name')
 
     action = '/'.join(['repos', org_name, repo_name])
-    command = 'issues/' + str(issue_number)
+    command = 'issues/' + six.text_type(issue_number)
 
     ret = {}
     issue_data = _query(profile, action=action, command=command)
@@ -399,7 +416,7 @@ def get_issue_comments(issue_number,
         repo_name = _get_config_value(profile, 'repo_name')
 
     action = '/'.join(['repos', org_name, repo_name])
-    command = '/'.join(['issues', str(issue_number), 'comments'])
+    command = '/'.join(['issues', six.text_type(issue_number), 'comments'])
 
     args = {}
     if since:
@@ -684,7 +701,7 @@ def get_milestone(number=None,
 
     action = '/'.join(['repos', org_name, repo_name])
     if number:
-        command = 'milestones/' + str(number)
+        command = 'milestones/' + six.text_type(number)
         milestone_data = _query(profile, action=action, command=command)
         milestone_id = milestone_data.get('id')
         if output == 'full':
@@ -706,7 +723,33 @@ def get_milestone(number=None,
     return ret
 
 
-def get_repo_info(repo_name, profile='github'):
+def _repo_to_dict(repo):
+    ret = {}
+    ret['id'] = repo.id
+    ret['name'] = repo.name
+    ret['full_name'] = repo.full_name
+    ret['owner'] = repo.owner.login
+    ret['private'] = repo.private
+    ret['html_url'] = repo.html_url
+    ret['description'] = repo.description
+    ret['fork'] = repo.fork
+    ret['homepage'] = repo.homepage
+    ret['size'] = repo.size
+    ret['stargazers_count'] = repo.stargazers_count
+    ret['watchers_count'] = repo.watchers_count
+    ret['language'] = repo.language
+    ret['open_issues_count'] = repo.open_issues_count
+    ret['forks'] = repo.forks
+    ret['open_issues'] = repo.open_issues
+    ret['watchers'] = repo.watchers
+    ret['default_branch'] = repo.default_branch
+    ret['has_issues'] = repo.has_issues
+    ret['has_wiki'] = repo.has_wiki
+    ret['has_downloads'] = repo.has_downloads
+    return ret
+
+
+def get_repo_info(repo_name, profile='github', ignore_cache=False):
     '''
     Return information for a given repo.
 
@@ -725,46 +768,80 @@ def get_repo_info(repo_name, profile='github'):
         salt myminion github.get_repo_info salt
         salt myminion github.get_repo_info salt profile='my-github-profile'
     '''
-    ret = {}
+
+    org_name = _get_config_value(profile, 'org_name')
+    key = "github.{0}:{1}:repo_info".format(
+        _get_config_value(profile, 'org_name'),
+        repo_name.lower()
+    )
+
+    if key not in __context__ or ignore_cache:
+        client = _get_client(profile)
+        try:
+            repo = client.get_repo('/'.join([org_name, repo_name]))
+            if not repo:
+                return {}
+
+            # client.get_repo can return a github.Repository.Repository object,
+            # even if the repo is invalid. We need to catch the exception when
+            # we try to perform actions on the repo object, rather than above
+            # the if statement.
+            ret = _repo_to_dict(repo)
+
+            __context__[key] = ret
+        except github.UnknownObjectException:
+            raise CommandExecutionError(
+                'The \'{0}\' repository under the \'{1}\' organization could not '
+                'be found.'.format(
+                    repo_name,
+                    org_name
+                )
+            )
+    return __context__[key]
+
+
+def get_repo_teams(repo_name, profile='github'):
+    '''
+    Return teams belonging to a repository.
+
+    .. versionadded:: 2017.7.0
+
+    repo_name
+        The name of the repository from which to retrieve teams.
+
+    profile
+        The name of the profile configuration to use. Defaults to ``github``.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt myminion github.get_repo_teams salt
+        salt myminion github.get_repo_teams salt profile='my-github-profile'
+    '''
+    ret = []
     org_name = _get_config_value(profile, 'org_name')
     client = _get_client(profile)
 
     try:
         repo = client.get_repo('/'.join([org_name, repo_name]))
-        if repo:
-            # client.get_repo can return a github.Repository.Repository object,
-            # even if the repo is invalid. We need to catch the exception when
-            # we try to perform actions on the repo object, rather than above
-            # the if statement.
-            ret['id'] = repo.id
-
-            ret['name'] = repo.name
-            ret['full_name'] = repo.full_name
-            ret['owner'] = repo.owner.login
-            ret['private'] = repo.private
-            ret['html_url'] = repo.html_url
-            ret['description'] = repo.description
-            ret['fork'] = repo.fork
-            ret['homepage'] = repo.homepage
-            ret['size'] = repo.size
-            ret['stargazers_count'] = repo.stargazers_count
-            ret['watchers_count'] = repo.watchers_count
-            ret['language'] = repo.language
-            ret['open_issues_count'] = repo.open_issues_count
-            ret['forks'] = repo.forks
-            ret['open_issues'] = repo.open_issues
-            ret['watchers'] = repo.watchers
-            ret['default_branch'] = repo.default_branch
-            ret['has_issues'] = repo.has_issues
-            ret['has_wiki'] = repo.has_wiki
-            ret['has_downloads'] = repo.has_downloads
     except github.UnknownObjectException:
         raise CommandExecutionError(
             'The \'{0}\' repository under the \'{1}\' organization could not '
-            'be found.'.format(
-                repo_name,
-                org_name
-            )
+            'be found.'.format(repo_name, org_name)
+        )
+    try:
+        teams = repo.get_teams()
+        for team in teams:
+            ret.append({
+                'id': team.id,
+                'name': team.name,
+                'permission': team.permission
+            })
+    except github.UnknownObjectException:
+        raise CommandExecutionError(
+            'Unable to retrieve teams for repository \'{0}\' under the \'{1}\' '
+            'organization.'.format(repo_name, org_name)
         )
     return ret
 
@@ -920,8 +997,8 @@ def add_repo(name,
             input=parameters
         )
         return True
-    except github.GithubException as e:
-        log.exception('Error creating a repo: {0}'.format(str(e)))
+    except github.GithubException:
+        log.exception('Error creating a repo')
         return False
 
 
@@ -969,6 +1046,17 @@ def edit_repo(name,
 
     .. versionadded:: 2016.11.0
     '''
+
+    try:
+        allow_private_change = _get_config_value(profile, 'allow_repo_privacy_changes')
+    except CommandExecutionError:
+        allow_private_change = False
+
+    if private is not None and not allow_private_change:
+        raise CommandExecutionError("The private field is set to be changed for "
+                                    "repo {0} but allow_repo_privacy_changes "
+                                    "disallows this.".format(name))
+
     try:
         client = _get_client(profile)
         organization = client.get_organization(
@@ -994,9 +1082,10 @@ def edit_repo(name,
             repo.url,
             input=parameters
         )
+        get_repo_info(name, profile=profile, ignore_cache=True)  # Refresh cache
         return True
-    except github.GithubException as e:
-        log.exception('Error editing a repo: {0}'.format(str(e)))
+    except github.GithubException:
+        log.exception('Error editing a repo')
         return False
 
 
@@ -1020,7 +1109,7 @@ def remove_repo(name, profile="github"):
     '''
     repo_info = get_repo_info(name, profile=profile)
     if not repo_info:
-        log.error('Repo {0} to be removed does not exist.'.format(name))
+        log.error('Repo %s to be removed does not exist.', name)
         return False
     try:
         client = _get_client(profile)
@@ -1031,8 +1120,8 @@ def remove_repo(name, profile="github"):
         repo.delete()
         _get_repos(profile=profile, ignore_cache=True)  # refresh cache
         return True
-    except github.GithubException as e:
-        log.exception('Error deleting a repo: {0}'.format(str(e)))
+    except github.GithubException:
+        log.exception('Error deleting a repo')
         return False
 
 
@@ -1116,8 +1205,8 @@ def add_team(name,
         )
         list_teams(ignore_cache=True)  # Refresh cache
         return True
-    except github.GithubException as e:
-        log.exception('Error creating a team: {0}'.format(str(e)))
+    except github.GithubException:
+        log.exception('Error creating a team')
         return False
 
 
@@ -1155,7 +1244,7 @@ def edit_team(name,
     '''
     team = get_team(name, profile=profile)
     if not team:
-        log.error('Team {0} does not exist'.format(name))
+        log.error('Team %s does not exist', name)
         return False
     try:
         client = _get_client(profile)
@@ -1167,9 +1256,9 @@ def edit_team(name,
         parameters = {}
         if name is not None:
             parameters['name'] = name
-        if 'description' is not None:
+        if description is not None:
             parameters['description'] = description
-        if 'privacy' is not None:
+        if privacy is not None:
             parameters['privacy'] = privacy
         if permission is not None:
             parameters['permission'] = permission
@@ -1180,8 +1269,8 @@ def edit_team(name,
             input=parameters
         )
         return True
-    except UnknownObjectException as e:
-        log.exception('Resource not found: {0}'.format(team['id']))
+    except UnknownObjectException:
+        log.exception('Resource not found')
         return False
 
 
@@ -1205,7 +1294,7 @@ def remove_team(name, profile="github"):
     '''
     team_info = get_team(name, profile=profile)
     if not team_info:
-        log.error('Team {0} to be removed does not exist.'.format(name))
+        log.error('Team %s to be removed does not exist.', name)
         return False
     try:
         client = _get_client(profile)
@@ -1215,14 +1304,15 @@ def remove_team(name, profile="github"):
         team = organization.get_team(team_info['id'])
         team.delete()
         return list_teams(ignore_cache=True, profile=profile).get(name) is None
-    except github.GithubException as e:
-        log.exception('Error deleting a team: {0}'.format(str(e)))
+    except github.GithubException:
+        log.exception('Error deleting a team')
         return False
 
 
 def list_team_repos(team_name, profile="github", ignore_cache=False):
     '''
-    Gets the names of team repos in lower case.
+    Gets the repo details for a given team as a dict from repo_name to repo details.
+    Note that repo names are always in lower case.
 
     team_name
         The name of the team from which to list repos.
@@ -1243,7 +1333,7 @@ def list_team_repos(team_name, profile="github", ignore_cache=False):
     '''
     cached_team = get_team(team_name, profile=profile)
     if not cached_team:
-        log.error('Team {0} does not exist.'.format(team_name))
+        log.error('Team %s does not exist.', team_name)
         return False
 
     # Return from cache if available
@@ -1256,17 +1346,28 @@ def list_team_repos(team_name, profile="github", ignore_cache=False):
             _get_config_value(profile, 'org_name')
         )
         team = organization.get_team(cached_team['id'])
-    except UnknownObjectException as e:
-        log.exception('Resource not found: {0}'.format(cached_team['id']))
+    except UnknownObjectException:
+        log.exception('Resource not found: %s', cached_team['id'])
     try:
-        cached_team['repos'] = [repo.name.lower() for repo in team.get_repos()]
-        return cached_team['repos']
-    except UnknownObjectException as e:
-        log.exception('Resource not found: {0}'.format(cached_team['id']))
+        repos = {}
+        for repo in team.get_repos():
+            permission = 'pull'
+            if repo.permissions.admin:
+                permission = 'admin'
+            elif repo.permissions.push:
+                permission = 'push'
+
+            repos[repo.name.lower()] = {
+                'permission': permission
+            }
+        cached_team['repos'] = repos
+        return repos
+    except UnknownObjectException:
+        log.exception('Resource not found: %s', cached_team['id'])
         return []
 
 
-def add_team_repo(repo_name, team_name, profile="github"):
+def add_team_repo(repo_name, team_name, profile="github", permission=None):
     '''
     Adds a repository to a team with team_name.
 
@@ -1279,6 +1380,13 @@ def add_team_repo(repo_name, team_name, profile="github"):
     profile
         The name of the profile configuration to use. Defaults to ``github``.
 
+    permission
+        The permission for team members within the repository, can be 'pull',
+        'push' or 'admin'. If not specified, the default permission specified on
+        the team will be used.
+
+        .. versionadded:: 2017.7.0
+
     CLI Example:
 
     .. code-block:: bash
@@ -1289,7 +1397,7 @@ def add_team_repo(repo_name, team_name, profile="github"):
     '''
     team = get_team(team_name, profile=profile)
     if not team:
-        log.error('Team {0} does not exist'.format(team_name))
+        log.error('Team %s does not exist', team_name)
         return False
     try:
         client = _get_client(profile)
@@ -1298,11 +1406,21 @@ def add_team_repo(repo_name, team_name, profile="github"):
         )
         team = organization.get_team(team['id'])
         repo = organization.get_repo(repo_name)
-    except UnknownObjectException as e:
-        log.exception('Resource not found: {0}'.format(team['id']))
+    except UnknownObjectException:
+        log.exception('Resource not found: %s', team['id'])
         return False
-    team.add_to_repos(repo)
-    return repo_name in list_team_repos(team_name, profile=profile, ignore_cache=True)
+    params = None
+    if permission is not None:
+        params = {'permission': permission}
+
+    headers, data = team._requester.requestJsonAndCheck(
+        "PUT",
+        team.url + "/repos/" + repo._identity,
+        input=params
+    )
+    # Try to refresh cache
+    list_team_repos(team_name, profile=profile, ignore_cache=True)
+    return True
 
 
 def remove_team_repo(repo_name, team_name, profile="github"):
@@ -1328,7 +1446,7 @@ def remove_team_repo(repo_name, team_name, profile="github"):
     '''
     team = get_team(team_name, profile=profile)
     if not team:
-        log.error('Team {0} does not exist'.format(team_name))
+        log.error('Team %s does not exist', team_name)
         return False
     try:
         client = _get_client(profile)
@@ -1337,8 +1455,8 @@ def remove_team_repo(repo_name, team_name, profile="github"):
         )
         team = organization.get_team(team['id'])
         repo = organization.get_repo(repo_name)
-    except UnknownObjectException as e:
-        log.exception('Resource not found: {0}'.format(team['id']))
+    except UnknownObjectException:
+        log.exception('Resource not found: %s', team['id'])
         return False
     team.remove_from_repos(repo)
     return repo_name not in list_team_repos(team_name, profile=profile, ignore_cache=True)
@@ -1367,7 +1485,7 @@ def list_team_members(team_name, profile="github", ignore_cache=False):
     '''
     cached_team = get_team(team_name, profile=profile)
     if not cached_team:
-        log.error('Team {0} does not exist.'.format(team_name))
+        log.error('Team %s does not exist.', team_name)
         return False
     # Return from cache if available
     if cached_team.get('members') and not ignore_cache:
@@ -1379,14 +1497,14 @@ def list_team_members(team_name, profile="github", ignore_cache=False):
             _get_config_value(profile, 'org_name')
         )
         team = organization.get_team(cached_team['id'])
-    except UnknownObjectException as e:
-        log.exception('Resource not found: {0}'.format(cached_team['id']))
+    except UnknownObjectException:
+        log.exception('Resource not found: %s', cached_team['id'])
     try:
         cached_team['members'] = [member.login.lower()
                                   for member in team.get_members()]
         return cached_team['members']
-    except UnknownObjectException as e:
-        log.exception('Resource not found: {0}'.format(cached_team['id']))
+    except UnknownObjectException:
+        log.exception('Resource not found: %s', cached_team['id'])
         return []
 
 
@@ -1477,7 +1595,7 @@ def add_team_member(name, team_name, profile="github"):
     '''
     team = get_team(team_name, profile=profile)
     if not team:
-        log.error('Team {0} does not exist'.format(team_name))
+        log.error('Team %s does not exist', team_name)
         return False
     try:
         client = _get_client(profile)
@@ -1486,8 +1604,8 @@ def add_team_member(name, team_name, profile="github"):
         )
         team = organization.get_team(team['id'])
         member = client.get_user(name)
-    except UnknownObjectException as e:
-        log.exception('Resource not found: {0}'.format(team['id']))
+    except UnknownObjectException:
+        log.exception('Resource not found: %s', team['id'])
         return False
 
     try:
@@ -1499,10 +1617,10 @@ def add_team_member(name, team_name, profile="github"):
             input={'role': 'member'},
             parameters={'role': 'member'}
         )
-    except github.GithubException as e:
-        log.exception('Error in adding a member to a team: {0}'.format(str(e)))
+    except github.GithubException:
+        log.exception('Error in adding a member to a team')
         return False
-    return team.has_in_members(member)
+    return True
 
 
 def remove_team_member(name, team_name, profile="github"):
@@ -1528,7 +1646,7 @@ def remove_team_member(name, team_name, profile="github"):
     '''
     team = get_team(team_name, profile=profile)
     if not team:
-        log.error('Team {0} does not exist'.format(team_name))
+        log.error('Team %s does not exist', team_name)
         return False
     try:
         client = _get_client(profile)
@@ -1538,8 +1656,8 @@ def remove_team_member(name, team_name, profile="github"):
         team = organization.get_team(team['id'])
         member = client.get_user(name)
 
-    except UnknownObjectException as e:
-        log.exception('Resource not found: {0}'.format(team['id']))
+    except UnknownObjectException:
+        log.exception('Resource not found: %s', team['id'])
         return False
 
     if not hasattr(team, 'remove_from_members'):
@@ -1580,18 +1698,140 @@ def list_teams(profile="github", ignore_cache=False):
         teams_data = organization.get_teams()
         teams = {}
         for team in teams_data:
+            # Note that _rawData is used to access some properties here as they
+            # are not exposed in older versions of PyGithub. It's VERY important
+            # to use team._rawData instead of team.raw_data, as the latter forces
+            # an API call to retrieve team details again.
             teams[team.name] = {
                 'id': team.id,
                 'slug': team.slug,
-                'description': team.raw_data['description'],
+                'description': team._rawData['description'],
                 'permission': team.permission,
-                'privacy': team.raw_data['privacy']
+                'privacy': team._rawData['privacy']
             }
         __context__[key] = teams
 
     return __context__[key]
 
 
+def get_prs(repo_name=None,
+            profile='github',
+            state='open',
+            head=None,
+            base=None,
+            sort='created',
+            direction='desc',
+            output='min',
+            per_page=None):
+    '''
+    Returns information for all pull requests in a given repository, based on
+    the search options provided.
+
+    .. versionadded:: 2017.7.0
+
+    repo_name
+        The name of the repository for which to list pull requests. This
+        argument is required, either passed via the CLI, or defined in the
+        configured profile. A ``repo_name`` passed as a CLI argument will
+        override the ``repo_name`` defined in the configured profile, if
+        provided.
+
+    profile
+        The name of the profile configuration to use. Defaults to ``github``.
+
+    state
+        Indicates the state of the pull requests to return. Can be either
+        ``open``, ``closed``, or ``all``. Default is ``open``.
+
+    head
+        Filter pull requests by head user and branch name in the format of
+        ``user:ref-name``. Example: ``'github:new-script-format'``. Default
+        is ``None``.
+
+    base
+        Filter pulls by base branch name. Example: ``gh-pages``. Default is
+        ``None``.
+
+    sort
+        What to sort results by. Can be either ``created``, ``updated``,
+        ``popularity`` (comment count), or ``long-running`` (age, filtering
+        by pull requests updated within the last month). Default is ``created``.
+
+    direction
+        The direction of the sort. Can be either ``asc`` or ``desc``. Default
+        is ``desc``.
+
+    output
+        The amount of data returned by each pull request. Defaults to ``min``.
+        Change to ``full`` to see all pull request output.
+
+    per_page
+        GitHub paginates data in their API calls. Use this value to increase or
+        decrease the number of pull requests gathered from GitHub, per page. If
+        not set, GitHub defaults are used. Maximum is 100.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt myminion github.get_prs
+        salt myminion github.get_prs base=2016.11
+    '''
+    org_name = _get_config_value(profile, 'org_name')
+    if repo_name is None:
+        repo_name = _get_config_value(profile, 'repo_name')
+
+    action = '/'.join(['repos', org_name, repo_name])
+    args = {}
+
+    # Build API arguments, as necessary.
+    if head:
+        args['head'] = head
+    if base:
+        args['base'] = base
+    if per_page:
+        args['per_page'] = per_page
+
+    # Only pass the following API args if they're not the defaults listed.
+    if state and state != 'open':
+        args['state'] = state
+    if sort and sort != 'created':
+        args['sort'] = sort
+    if direction and direction != 'desc':
+        args['direction'] = direction
+
+    ret = {}
+    prs = _query(profile, action=action, command='pulls', args=args)
+
+    for pr_ in prs:
+        pr_id = pr_.get('id')
+        if output == 'full':
+            ret[pr_id] = pr_
+        else:
+            ret[pr_id] = _format_pr(pr_)
+
+    return ret
+
+
+def _format_pr(pr_):
+    '''
+    Helper function to format API return information into a more manageable
+    and useful dictionary for pull request information.
+
+    pr_
+        The pull request to format.
+    '''
+    ret = {'id': pr_.get('id'),
+           'pr_number': pr_.get('number'),
+           'state': pr_.get('state'),
+           'title': pr_.get('title'),
+           'user': pr_.get('user').get('login'),
+           'html_url': pr_.get('html_url'),
+           'base_branch': pr_.get('base').get('ref')}
+
+    return ret
+
+
 def _format_issue(issue):
     '''
     Helper function to format API return information into a more manageable
@@ -1648,7 +1888,7 @@ def _query(profile,
     if command:
         url += '/{0}'.format(command)
 
-    log.debug('GitHub URL: {0}'.format(url))
+    log.debug('GitHub URL: %s', url)
 
     if 'access_token' not in args.keys():
         args['access_token'] = _get_config_value(profile, 'token')
@@ -1686,11 +1926,8 @@ def _query(profile,
                                        hide_fields=['access_token'],
                                        opts=__opts__,
                                        )
-        log.debug(
-            'GitHub Response Status Code: {0}'.format(
-                result['status']
-            )
-        )
+        log.debug('GitHub Response Status Code: %s',
+                  result['status'])
 
         if result['status'] == 200:
             if isinstance(result['dict'], dict):
