@@ -17,12 +17,14 @@ import salt.utils.path
 import salt.utils.platform
 import salt.utils.mount
 import salt.utils.stringutils
+from salt.utils.odict import OrderedDict
 from salt.exceptions import CommandNotFoundError, CommandExecutionError
 
 # Import 3rd-party libs
 from salt.ext import six
 from salt.ext.six.moves import filter, zip  # pylint: disable=import-error,redefined-builtin
 
+
 # Set up logger
 log = logging.getLogger(__name__)
 
@@ -61,7 +63,9 @@ def _active_mountinfo(ret):
         msg = 'File not readable {0}'
         raise CommandExecutionError(msg.format(filename))
 
-    blkid_info = __salt__['disk.blkid']()
+    if 'disk.blkid' not in __context__:
+        __context__['disk.blkid'] = __salt__['disk.blkid']()
+    blkid_info = __context__['disk.blkid']
 
     with salt.utils.files.fopen(filename) as ifile:
         for line in ifile:
@@ -121,18 +125,30 @@ def _active_mounts_aix(ret):
     '''
     for line in __salt__['cmd.run_stdout']('mount -p').split('\n'):
         comps = re.sub(r"\s+", " ", line).split()
-        if comps and comps[0] == 'node' or comps[0] == '--------':
-            continue
-        if len(comps) < 8:
-            ret[comps[1]] = {'device': comps[0],
-                             'fstype': comps[2],
-                             'opts': _resolve_user_group_names(comps[6].split(','))}
-        else:
-            ret[comps[2]] = {'node': comps[0],
-                             'device': comps[1],
-                             'fstype': comps[3],
-                             'opts': _resolve_user_group_names(comps[7].split(','))}
-
+        if comps:
+            if comps[0] == 'node' or comps[0] == '--------':
+                continue
+            comps_len = len(comps)
+            if line.startswith((' ', '\t')):
+                curr_opts = _resolve_user_group_names(comps[6].split(',')) if 7 == comps_len else []
+                if curr_opts:
+                    ret[comps[1]] = {'device': comps[0],
+                                    'fstype': comps[2],
+                                    'opts': curr_opts}
+                else:
+                    ret[comps[1]] = {'device': comps[0],
+                                    'fstype': comps[2]}
+            else:
+                curr_opts = _resolve_user_group_names(comps[7].split(',')) if 8 == comps_len else []
+                if curr_opts:
+                    ret[comps[2]] = {'node': comps[0],
+                                    'device': comps[1],
+                                    'fstype': comps[3],
+                                    'opts': curr_opts}
+                else:
+                    ret[comps[2]] = {'node': comps[0],
+                                    'device': comps[1],
+                                    'fstype': comps[3]}
     return ret
 
 
@@ -212,6 +228,7 @@ def _resolve_user_group_names(opts):
                 if _info and _param in _info:
                     _id = _info[_param]
             opts[ind] = _param + '=' + six.text_type(_id)
+        opts[ind] = opts[ind].replace('\\040', '\\ ')
     return opts
 
 
@@ -228,7 +245,7 @@ def active(extended=False):
     ret = {}
     if __grains__['os'] == 'FreeBSD':
         _active_mounts_freebsd(ret)
-    elif __grains__['kernel'] == 'AIX':
+    elif 'AIX' in __grains__['kernel']:
         _active_mounts_aix(ret)
     elif __grains__['kernel'] == 'SunOS':
         _active_mounts_solaris(ret)
@@ -276,9 +293,11 @@ class _fstab_entry(object):
             raise cls.ParseError("Comment!")
 
         comps = line.split()
-        if len(comps) != 6:
+        if len(comps) < 4 or len(comps) > 6:
             raise cls.ParseError("Invalid Entry!")
 
+        comps.extend(['0'] * (len(keys) - len(comps)))
+
         return dict(zip(keys, comps))
 
     @classmethod
@@ -420,9 +439,145 @@ class _vfstab_entry(object):
         return True
 
 
+class _FileSystemsEntry(object):
+    '''
+    Utility class for manipulating filesystem entries. Primarily we're parsing,
+    formatting, and comparing lines. Parsing emits dicts expected from
+    fstab() or raises a ValueError.
+
+    Note: We'll probably want to use os.normpath and os.normcase on 'name'
+    '''
+
+    class ParseError(ValueError):
+        '''
+        Error raised when a line isn't parsible as an fstab entry
+        '''
+
+    filesystems_keys = ('device', 'name', 'fstype', 'vfstype', 'opts', 'mount')
+
+    # preserve data format of filesystems
+    compatibility_keys = ('dev', 'dev', 'name', 'fstype', 'vfstype', 'opts', 'mount', 'type', 'vfs', 'account', 'boot', 'check', 'free', 'nodename', 'quota', 'size', 'vol', 'log')
+
+    @classmethod
+    def dict_from_lines(cls, lines, keys=filesystems_keys):
+        if len(lines) < 2:
+            raise ValueError('Invalid number of lines: {0}'.format(lines))
+        if not keys:
+            # if empty force default filesystems_keys
+            keys = _FileSystemsEntry.filesystems_keys
+        elif len(keys) < 6:
+            raise ValueError('Invalid key name array: {0}'.format(keys))
+
+        blk_lines = lines
+        orddict = OrderedDict()
+        orddict['name'] = blk_lines[0].split(':')[0].strip()
+        blk_lines.pop(0)
+        for line in blk_lines:
+            if line.startswith('#'):
+                raise cls.ParseError("Comment!")
+
+            comps = line.split('= ')
+            if len(comps) != 2:
+                raise cls.ParseError("Invalid Entry!")
+
+            key_name = comps[0].strip()
+            if key_name in keys:
+                orddict[key_name] = comps[1].strip()
+            else:
+                raise ValueError('Invalid name for use in filesystems: {0}'.format(key_name))
+
+        return orddict
+
+    @classmethod
+    def dict_from_cmd_line(cls, ipargs, keys):
+        cmdln_dict = ipargs
+        if keys:
+            for key, value in keys:
+                # ignore unknown or local scope keys
+                if key.startswith('__'):
+                    continue
+                if key in _FileSystemsEntry.compatibility_keys:
+                    cmdln_dict[key] = value
+
+        return cmdln_dict
+
+    @classmethod
+    def from_line(cls, *args, **kwargs):
+        return cls(** cls.dict_from_cmd_line(*args, **kwargs))
+
+    @classmethod
+    def dict_to_lines(cls, fsys_dict_entry):
+        entry = fsys_dict_entry
+        strg_out = entry['name'] + ':' + os.linesep
+        for k, v in six.viewitems(entry):
+            if 'name' not in k:
+                strg_out += '\t{0}\t\t= {1}'.format(k, v) + os.linesep
+        strg_out += os.linesep
+        return six.text_type(strg_out)
+
+    def dict_from_entry(self):
+        ret = OrderedDict()
+        ret[self.criteria['name']] = self.criteria
+        return ret
+
+    def __str__(self):
+        '''
+        String value, only works for full repr
+        '''
+        return self.dict_to_lines(self.criteria)
+
+    def __repr__(self):
+        '''
+        Always works
+        '''
+        return repr(self.criteria)
+
+    def pick(self, keys):
+        '''
+        Returns an instance with just those keys
+        '''
+        subset = dict([(key, self.criteria[key]) for key in keys])
+        return self.__class__(**subset)
+
+    def __init__(self, **criteria):
+        '''
+        Store non-empty, non-null values to use as filter
+        '''
+        items = [key_value for key_value in six.iteritems(criteria) if key_value[1] is not None]
+        items = [(key_value1[0], six.text_type(key_value1[1])) for key_value1 in items]
+        self.criteria = OrderedDict(items)
+
+    @staticmethod
+    def norm_path(path):
+        '''
+        Resolve equivalent paths equivalently
+        '''
+        return os.path.normcase(os.path.normpath(path))
+
+    def match(self, fsys_view):
+        '''
+        Compare potentially partial criteria against built filesystems entry dictionary
+        '''
+        evalue_dict = fsys_view[1]
+        for key, value in six.viewitems(self.criteria):
+            if key in evalue_dict:
+                if evalue_dict[key] != value:
+                    return False
+            else:
+                return False
+        return True
+
+    def __getitem__(self, key):
+        '''
+        Return value for input key
+        '''
+        return self.criteria[key]
+
+
 def fstab(config='/etc/fstab'):
     '''
     .. versionchanged:: 2016.3.2
+
     List the contents of the fstab
 
     CLI Example:
@@ -465,6 +620,7 @@ def fstab(config='/etc/fstab'):
 def vfstab(config='/etc/vfstab'):
     '''
     .. versionadded:: 2016.3.2
+
     List the contents of the vfstab
 
     CLI Example:
@@ -480,6 +636,7 @@ def vfstab(config='/etc/vfstab'):
 def rm_fstab(name, device, config='/etc/fstab'):
     '''
     .. versionchanged:: 2016.3.2
+
     Remove the mount point from the fstab
 
     CLI Example:
@@ -531,6 +688,7 @@ def rm_fstab(name, device, config='/etc/fstab'):
 def rm_vfstab(name, device, config='/etc/vfstab'):
     '''
     .. versionadded:: 2016.3.2
+
     Remove the mount point from the vfstab
 
     CLI Example:
@@ -574,7 +732,7 @@ def set_fstab(
         'name': name,
         'device': device.replace('\\ ', '\\040'),
         'fstype': fstype,
-        'opts': opts,
+        'opts': opts.replace('\\ ', '\\040'),
         'dump': dump,
         'pass_num': pass_num,
     }
@@ -1040,7 +1198,7 @@ def mount(name, device, mkmnt=False, fstype='', opts='defaults', user=None, util
         return False
 
     # Darwin doesn't expect defaults when mounting without other options
-    if 'defaults' in opts and __grains__['os'] in ['MacOS', 'Darwin']:
+    if 'defaults' in opts and __grains__['os'] in ['MacOS', 'Darwin', 'AIX']:
         opts = None
 
     if isinstance(opts, six.string_types):
@@ -1053,8 +1211,16 @@ def mount(name, device, mkmnt=False, fstype='', opts='defaults', user=None, util
     if opts is not None:
         lopts = ','.join(opts)
         args = '-o {0}'.format(lopts)
+
     if fstype:
-        args += ' -t {0}'.format(fstype)
+        # use of fstype on AIX differs from typical Linux use of -t
+        # functionality AIX uses -v vfsname, -t fstype mounts all with
+        # fstype in /etc/filesystems
+        if 'AIX' in __grains__['os']:
+            args += ' -v {0}'.format(fstype)
+        else:
+            args += ' -t {0}'.format(fstype)
+
     cmd = 'mount {0} {1} {2} '.format(args, device, name)
     out = __salt__['cmd.run_all'](cmd, runas=user, python_shell=False)
     if out['retcode']:
@@ -1080,6 +1246,10 @@ def remount(name, device, mkmnt=False, fstype='', opts='defaults', user=None):
         if fstype == 'smbfs':
             force_mount = True
 
+    if 'AIX' in __grains__['os']:
+        if opts == 'defaults':
+            opts = ''
+
     if isinstance(opts, six.string_types):
         opts = opts.split(',')
     mnts = active()
@@ -1092,8 +1262,16 @@ def remount(name, device, mkmnt=False, fstype='', opts='defaults', user=None):
             umount(name, device, user=user)
         lopts = ','.join(opts)
         args = '-o {0}'.format(lopts)
-        if fstype:
+
+        # use of fstype on AIX differs from typical Linux use of -t functionality
+        # AIX uses -v vfsname, -t fstype mounts all with fstype in /etc/filesystems
+        if 'AIX' in __grains__['os']:
+            if fstype:
+                args += ' -v {0}'.format(fstype)
+            args += ' -o remount'
+        else:
             args += ' -t {0}'.format(fstype)
+
         if __grains__['os'] not in ['OpenBSD', 'MacOS', 'Darwin'] or force_mount:
             cmd = 'mount {0} {1} {2} '.format(args, device, name)
         else:
@@ -1186,6 +1364,17 @@ def swaps():
                              'size': int(comps[3]),
                              'used': (int(comps[3]) - int(comps[4])),
                              'priority': '-'}
+    elif 'AIX' in __grains__['kernel']:
+        for line in __salt__['cmd.run_stdout']('swap -l').splitlines():
+            if line.startswith('device'):
+                continue
+            comps = line.split()
+
+            # AIX uses MB for units
+            ret[comps[0]] = {'type': 'device',
+                             'size': int(comps[3][:-2]) * 1024,
+                             'used': (int(comps[3][:-2]) - int(comps[4][:-2])) * 1024,
+                             'priority': '-'}
     elif __grains__['os'] != 'OpenBSD':
         with salt.utils.files.fopen('/proc/swaps') as fp_:
             for line in fp_:
@@ -1238,7 +1427,7 @@ def swapon(name, priority=None):
             return False
     else:
         cmd = 'swapon {0}'.format(name)
-        if priority:
+        if priority and 'AIX' not in __grains__['kernel']:
             cmd += ' -p {0}'.format(priority)
         __salt__['cmd.run'](cmd, python_shell=False)
 
@@ -1387,3 +1576,278 @@ def delete_mount_cache(real_name):
                 if not cache_write:
                     raise CommandExecutionError('Unable to write mount cache.')
     return True
+
+
+def _filesystems(config='/etc/filesystems', leading_key=True):
+    '''
+    Return the contents of the filesystems in an OrderedDict
+
+    config
+        File containing filesystem infomation
+
+    leading_key
+        True    return dictionary keyed by 'name' and value as dictionary with other keys, values (name excluded)
+                OrderedDict({ '/dir' : OrderedDict({'dev': '/dev/hd8', .... }}))
+
+        False   return dictionary  keyed by 'name' and value as dictionary with all keys, values (name included)
+                OrderedDict({ '/dir' : OrderedDict({'name': '/dir', 'dev': '/dev/hd8', ... })})
+    '''
+    ret = OrderedDict()
+    lines = []
+    parsing_block = False
+    if not os.path.isfile(config) or 'AIX' not in __grains__['kernel']:
+        return ret
+
+    # read in block of filesystems, block starts with '/' till empty line
+    with salt.utils.files.fopen(config) as ifile:
+        for line in ifile:
+            line = salt.utils.stringutils.to_unicode(line)
+
+            # skip till first entry
+            if not line.startswith('/') and not parsing_block:
+                continue
+
+            if line.startswith('/'):
+                parsing_block = True
+                lines.append(line)
+            elif not line.split():
+                parsing_block = False
+                try:
+                    entry = _FileSystemsEntry.dict_from_lines(
+                        lines,
+                        _FileSystemsEntry.compatibility_keys)
+                    lines = []
+                    if 'opts' in entry:
+                        entry['opts'] = entry['opts'].split(',')
+                    while entry['name'] in ret:
+                        entry['name'] += '_'
+
+                    if leading_key:
+                        ret[entry.pop('name')] = entry
+                    else:
+                        ret[entry['name']] = entry
+
+                except _FileSystemsEntry.ParseError:
+                    pass
+            else:
+                lines.append(line)
+
+    return ret
+
+
+def filesystems(config='/etc/filesystems'):
+    '''
+    .. versionadded:: 2018.3.3
+
+    List the contents of the filesystems
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' mount.filesystems
+    '''
+    ret = {}
+    if 'AIX' not in __grains__['kernel']:
+        return ret
+
+    ret_dict = _filesystems(config)
+    if ret_dict:
+        ret_key = next(iter(ret_dict.keys()))
+        ret = {ret_key: dict(ret_dict[ret_key])}
+
+    return ret
+
+
+def set_filesystems(
+        name,
+        device,
+        vfstype,
+        opts='-',
+        mount='true',
+        config='/etc/filesystems',
+        test=False,
+        match_on='auto',
+        **kwargs):
+    '''
+    .. versionadded:: 2018.3.3
+
+    Verify that this mount is represented in the filesystems, change the mount
+    to match the data passed, or add the mount if it is not present on AIX
+
+        Provide information if the path is mounted
+
+    :param name:          The name of the mount point where the device is mounted.
+    :param device:        The device that is being mounted.
+    :param vfstype:       The file system that is used (AIX has two fstypes, fstype and vfstype - similar to Linux fstype)
+    :param opts:          Additional options used when mounting the device.
+    :param mount:         Mount if not mounted, default True.
+    :param config:        Configuration file, default /etc/filesystems.
+    :param match:         File systems type to match on, default auto
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' mount.set_filesystems /mnt/foo /dev/sdz1 jfs2
+    '''
+    # Fix the opts type if it is a list
+    if isinstance(opts, list):
+        opts = ','.join(opts)
+
+    # preserve arguments for updating
+    entry_args = {
+        'name': name,
+        'dev': device.replace('\\ ', '\\040'),
+        'vfstype': vfstype,
+        'opts': opts,
+        'mount': mount,
+    }
+
+    view_lines = []
+    ret = None
+
+    if 'AIX' not in __grains__['kernel']:
+        return ret
+
+    # Transform match_on into list--items will be checked later
+    if isinstance(match_on, list):
+        pass
+    elif not isinstance(match_on, six.string_types):
+        raise CommandExecutionError('match_on must be a string or list of strings')
+    elif match_on == 'auto':
+        # Try to guess right criteria for auto....
+        # added IBM types from sys/vmount.h after btrfs
+        # NOTE: missing some special fstypes here
+        specialFSes = frozenset([
+            'none',
+            'tmpfs',
+            'sysfs',
+            'proc',
+            'fusectl',
+            'debugfs',
+            'securityfs',
+            'devtmpfs',
+            'cgroup',
+            'btrfs',
+            'cdrfs',
+            'procfs',
+            'jfs',
+            'jfs2',
+            'nfs',
+            'sfs',
+            'nfs3',
+            'cachefs',
+            'udfs',
+            'cifs',
+            'namefs',
+            'pmemfs',
+            'ahafs',
+            'nfs4',
+            'autofs',
+            'stnfs'])
+
+        if vfstype in specialFSes:
+            match_on = ['name']
+        else:
+            match_on = ['dev']
+    else:
+        match_on = [match_on]
+
+    # generate entry and criteria objects, handle invalid keys in match_on
+    entry_ip = _FileSystemsEntry.from_line(entry_args, kwargs)
+    try:
+        criteria = entry_ip.pick(match_on)
+
+    except KeyError:
+        filterFn = lambda key: key not in _FileSystemsEntry.compatibility_keys
+        invalid_keys = filter(filterFn, match_on)
+        raise CommandExecutionError('Unrecognized keys in match_on: "{0}"'.format(invalid_keys))
+
+    # parse file, use ret to cache status
+    if not os.path.isfile(config):
+        raise CommandExecutionError('Bad config file "{0}"'.format(config))
+
+    # read in block of filesystem, block starts with '/' till empty line
+    try:
+        fsys_filedict = _filesystems(config, False)
+        for fsys_view in six.viewitems(fsys_filedict):
+            if criteria.match(fsys_view):
+                ret = 'present'
+                if entry_ip.match(fsys_view):
+                    view_lines.append(fsys_view)
+                else:
+                    ret = 'change'
+                    kv = entry_ip['name']
+                    view_lines.append((kv, entry_ip))
+            else:
+                view_lines.append(fsys_view)
+
+    except (IOError, OSError) as exc:
+        raise CommandExecutionError('Couldn\'t read from {0}: {1}'.format(config, exc))
+
+    # add line if not present or changed
+    if ret is None:
+        for dict_view in six.viewitems(entry_ip.dict_from_entry()):
+            view_lines.append(dict_view)
+        ret = 'new'
+
+    if ret != 'present':  # ret in ['new', 'change']:
+        try:
+            with salt.utils.files.fopen(config, 'wb') as ofile:
+                # The line was changed, commit it!
+                for fsys_view in view_lines:
+                    entry = fsys_view[1]
+                    mystrg = _FileSystemsEntry.dict_to_lines(entry)
+                    ofile.writelines(salt.utils.data.encode(mystrg))
+        except (IOError, OSError):
+            raise CommandExecutionError('File not writable {0}'.format(config))
+
+    return ret
+
+
+def rm_filesystems(name, device, config='/etc/filesystems'):
+    '''
+    .. versionadded:: 2018.3.3
+
+    Remove the mount point from the filesystems
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' mount.rm_filesystems /mnt/foo /dev/sdg
+    '''
+    modified = False
+    view_lines = []
+
+    if 'AIX' not in __grains__['kernel']:
+        return modified
+
+    criteria = _FileSystemsEntry(name=name, dev=device)
+    try:
+        fsys_filedict = _filesystems(config, False)
+        for fsys_view in six.viewitems(fsys_filedict):
+            try:
+                if criteria.match(fsys_view):
+                    modified = True
+                else:
+                    view_lines.append(fsys_view)
+
+            except _FileSystemsEntry.ParseError:
+                view_lines.append(fsys_view)
+
+    except (IOError, OSError) as exc:
+        raise CommandExecutionError("Couldn't read from {0}: {1}".format(config, exc))
+
+    if modified:
+        try:
+            with salt.utils.files.fopen(config, 'wb') as ofile:
+                for fsys_view in view_lines:
+                    entry = fsys_view[1]
+                    mystrg = _FileSystemsEntry.dict_to_lines(entry)
+                    ofile.writelines(salt.utils.data.encode(mystrg))
+        except (IOError, OSError) as exc:
+            raise CommandExecutionError("Couldn't write to {0}: {1}".format(config, exc))
+
+    return modified
