@@ -21,6 +21,7 @@ import salt.utils.files
 import salt.utils.path
 import salt.utils.stringutils
 import salt.utils.decorators as decorators
+import salt.utils.versions
 from salt.exceptions import CommandExecutionError, SaltInvocationError
 
 # Import 3rd-party libs
@@ -387,6 +388,28 @@ def _validate_filetype(filetype):
     return True
 
 
+def _parse_protocol_port(name, protocol, port):
+    '''
+    .. versionadded:: 2019.2.0
+
+    Validates and parses the protocol and port/port range from the name
+    if both protocol and port are not provided.
+
+    If the name is in a valid format, the protocol and port are ignored if provided
+
+    Examples: tcp/8080 or udp/20-21
+    '''
+    protocol_port_pattern = r'^(tcp|udp)\/(([\d]+)\-?[\d]+)$'
+    name_parts = re.match(protocol_port_pattern, name)
+    if not name_parts:
+        name_parts = re.match(protocol_port_pattern, '{0}/{1}'.format(protocol, port))
+    if not name_parts:
+        raise SaltInvocationError(
+            'Invalid name "{0}" format and protocol and port not provided or invalid: "{1}" "{2}".'.format(
+                name, protocol, port))
+    return name_parts.group(1), name_parts.group(2)
+
+
 def _context_dict_to_string(context):
     '''
     .. versionadded:: 2017.7.0
@@ -456,7 +479,7 @@ def fcontext_get_policy(name, filetype=None, sel_type=None, sel_user=None, sel_l
     '''
     if filetype:
         _validate_filetype(filetype)
-    re_spacer = '[ ]{2,}'
+    re_spacer = '[ ]+'
     cmd_kwargs = {'spacer': re_spacer,
                   'filespec': re.escape(name),
                   'sel_user': sel_user or '[^:]+',
@@ -469,20 +492,22 @@ def fcontext_get_policy(name, filetype=None, sel_type=None, sel_user=None, sel_l
     current_entry_text = __salt__['cmd.shell'](cmd, ignore_retcode=True)
     if current_entry_text == '':
         return None
-    ret = {}
-    current_entry_list = re.split(re_spacer, current_entry_text)
-    ret['filespec'] = current_entry_list[0]
-    ret['filetype'] = current_entry_list[1]
-    ret.update(_context_string_to_dict(current_entry_list[2]))
+
+    parts = re.match(r'^({filespec}) +([a-z ]+) (.*)$'.format(**{'filespec': re.escape(name)}), current_entry_text)
+    ret = {
+        'filespec': parts.group(1).strip(),
+        'filetype': parts.group(2).strip(),
+    }
+    ret.update(_context_string_to_dict(parts.group(3).strip()))
+
     return ret
 
 
-def fcontext_add_or_delete_policy(action, name, filetype=None, sel_type=None, sel_user=None, sel_level=None):
+def fcontext_add_policy(name, filetype=None, sel_type=None, sel_user=None, sel_level=None):
     '''
-    .. versionadded:: 2017.7.0
+    .. versionadded:: 2019.2.0
 
-    Sets or deletes the SELinux policy for a given filespec and other
-    optional parameters.
+    Adds the SELinux policy for a given filespec and other optional parameters.
 
     Returns the result of the call to semanage.
 
@@ -510,14 +535,122 @@ def fcontext_add_or_delete_policy(action, name, filetype=None, sel_type=None, se
 
     CLI Example:
 
+    .. code-block:: bash
+
+        salt '*' selinux.fcontext_add_policy my-policy
+    '''
+    return _fcontext_add_or_delete_policy('add', name, filetype, sel_type, sel_user, sel_level)
+
+
+def fcontext_delete_policy(name, filetype=None, sel_type=None, sel_user=None, sel_level=None):
+    '''
+    .. versionadded:: 2019.2.0
+
+    Deletes the SELinux policy for a given filespec and other optional parameters.
+
+    Returns the result of the call to semanage.
+
+    Note that you don't have to remove an entry before setting a new
+    one for a given filespec and filetype, as adding one with semanage
+    automatically overwrites a previously configured SELinux context.
+
+    name
+        filespec of the file or directory. Regex syntax is allowed.
+
+    file_type
+        The SELinux filetype specification. Use one of [a, f, d, c, b,
+        s, l, p]. See also ``man semanage-fcontext``. Defaults to 'a'
+        (all files).
+
+    sel_type
+        SELinux context type. There are many.
+
+    sel_user
+        SELinux user. Use ``semanage login -l`` to determine which ones
+        are available to you.
+
+    sel_level
+        The MLS range of the SELinux context.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' selinux.fcontext_delete_policy my-policy
+    '''
+    return _fcontext_add_or_delete_policy('delete', name, filetype, sel_type, sel_user, sel_level)
+
+
+def fcontext_add_or_delete_policy(action, name, filetype=None, sel_type=None, sel_user=None, sel_level=None):
+    '''
+    .. versionadded:: 2017.7.0
+
+    Adds or deletes the SELinux policy for a given filespec and other optional parameters.
+
+    Returns the result of the call to semanage.
+
+    Note that you don't have to remove an entry before setting a new
+    one for a given filespec and filetype, as adding one with semanage
+    automatically overwrites a previously configured SELinux context.
+
+    .. warning::
+
+        Use :mod:`selinux.fcontext_add_policy()<salt.modules.selinux.fcontext_add_policy>`,
+        or :mod:`selinux.fcontext_delete_policy()<salt.modules.selinux.fcontext_delete_policy>`.
+
+    .. deprecated:: 2019.2.0
+
+    action
+        The action to perform. Either ``add`` or ``delete``.
+
+    name
+        filespec of the file or directory. Regex syntax is allowed.
+
+    file_type
+        The SELinux filetype specification. Use one of [a, f, d, c, b,
+        s, l, p]. See also ``man semanage-fcontext``. Defaults to 'a'
+        (all files).
+
+    sel_type
+        SELinux context type. There are many.
+
+    sel_user
+        SELinux user. Use ``semanage login -l`` to determine which ones
+        are available to you.
+
+    sel_level
+        The MLS range of the SELinux context.
+
+    CLI Example:
+
     .. code-block:: bash
 
         salt '*' selinux.fcontext_add_or_delete_policy add my-policy
     '''
+    salt.utils.versions.warn_until(
+        'Sodium',
+        'The \'selinux.fcontext_add_or_delete_policy\' module has been deprecated. Please use the '
+        '\'selinux.fcontext_add_policy\' and \'selinux.fcontext_delete_policy\' modules instead. '
+        'Support for the \'selinux.fcontext_add_or_delete_policy\' module will be removed in Salt '
+        '{version}.'
+    )
+    return _fcontext_add_or_delete_policy(action, name, filetype, sel_type, sel_user, sel_level)
+
+
+def _fcontext_add_or_delete_policy(action, name, filetype=None, sel_type=None, sel_user=None, sel_level=None):
+    '''
+    .. versionadded:: 2019.2.0
+
+    Performs the action as called from ``fcontext_add_policy`` or ``fcontext_delete_policy``.
+
+    Returns the result of the call to semanage.
+    '''
     if action not in ['add', 'delete']:
         raise SaltInvocationError('Actions supported are "add" and "delete", not "{0}".'.format(action))
     cmd = 'semanage fcontext --{0}'.format(action)
-    if filetype is not None:
+    # "semanage --ftype a" isn't valid on Centos 6,
+    # don't pass --ftype since "a" is the default filetype.
+    if filetype is not None and filetype != 'a':
         _validate_filetype(filetype)
         cmd += ' --ftype {0}'.format(filetype)
     if sel_type is not None:
@@ -559,7 +692,7 @@ def fcontext_apply_policy(name, recursive=False):
     .. versionadded:: 2017.7.0
 
     Applies SElinux policies to filespec using `restorecon [-R]
-    filespec`. Returns dict with changes if succesful, the output of
+    filespec`. Returns dict with changes if successful, the output of
     the restorecon command otherwise.
 
     name
@@ -599,3 +732,137 @@ def fcontext_apply_policy(name, recursive=False):
                 del new[key]
             ret['changes'].update({filespec: {'old': old, 'new': new}})
     return ret
+
+
+def port_get_policy(name, sel_type=None, protocol=None, port=None):
+    '''
+    .. versionadded:: 2019.2.0
+
+    Returns the current entry in the SELinux policy list as a
+    dictionary. Returns None if no exact match was found.
+
+    Returned keys are:
+
+    * sel_type (the selinux type)
+    * proto (the protocol)
+    * port (the port(s) and/or port range(s))
+
+    name
+        The protocol and port spec. Can be formatted as ``(tcp|udp)/(port|port-range)``.
+
+    sel_type
+        The SELinux Type.
+
+    protocol
+        The protocol for the port, ``tcp`` or ``udp``. Required if name is not formatted.
+
+    port
+        The port or port range. Required if name is not formatted.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' selinux.port_get_policy tcp/80
+        salt '*' selinux.port_get_policy foobar protocol=tcp port=80
+    '''
+    (protocol, port) = _parse_protocol_port(name, protocol, port)
+    re_spacer = '[ ]+'
+    re_sel_type = sel_type if sel_type else r'\w+'
+    cmd_kwargs = {'spacer': re_spacer,
+                  'sel_type': re_sel_type,
+                  'protocol': protocol,
+                  'port': port, }
+    cmd = 'semanage port -l | egrep ' + \
+          "'^{sel_type}{spacer}{protocol}{spacer}((.*)*)[ ]{port}($|,)'".format(**cmd_kwargs)
+    port_policy = __salt__['cmd.shell'](cmd, ignore_retcode=True)
+    if port_policy == '':
+        return None
+
+    parts = re.match(r'^(\w+)[ ]+(\w+)[ ]+([\d\-, ]+)', port_policy)
+    return {
+        'sel_type': parts.group(1).strip(),
+        'protocol': parts.group(2).strip(),
+        'port': parts.group(3).strip(), }
+
+
+def port_add_policy(name, sel_type=None, protocol=None, port=None, sel_range=None):
+    '''
+    .. versionadded:: 2019.2.0
+
+    Adds the SELinux policy for a given protocol and port.
+
+    Returns the result of the call to semanage.
+
+    name
+        The protocol and port spec. Can be formatted as ``(tcp|udp)/(port|port-range)``.
+
+    sel_type
+        The SELinux Type. Required.
+
+    protocol
+        The protocol for the port, ``tcp`` or ``udp``. Required if name is not formatted.
+
+    port
+        The port or port range. Required if name is not formatted.
+
+    sel_range
+        The SELinux MLS/MCS Security Range.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' selinux.port_add_policy add tcp/8080 http_port_t
+        salt '*' selinux.port_add_policy add foobar http_port_t protocol=tcp port=8091
+    '''
+    return _port_add_or_delete_policy('add', name, sel_type, protocol, port, sel_range)
+
+
+def port_delete_policy(name, protocol=None, port=None):
+    '''
+    .. versionadded:: 2019.2.0
+
+    Deletes the SELinux policy for a given protocol and port.
+
+    Returns the result of the call to semanage.
+
+    name
+        The protocol and port spec. Can be formatted as ``(tcp|udp)/(port|port-range)``.
+
+    protocol
+        The protocol for the port, ``tcp`` or ``udp``. Required if name is not formatted.
+
+    port
+        The port or port range. Required if name is not formatted.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' selinux.port_delete_policy tcp/8080
+        salt '*' selinux.port_delete_policy foobar protocol=tcp port=8091
+    '''
+    return _port_add_or_delete_policy('delete', name, None, protocol, port, None)
+
+
+def _port_add_or_delete_policy(action, name, sel_type=None, protocol=None, port=None, sel_range=None):
+    '''
+    .. versionadded:: 2019.2.0
+
+    Performs the action as called from ``port_add_policy`` or ``port_delete_policy``.
+
+    Returns the result of the call to semanage.
+    '''
+    if action not in ['add', 'delete']:
+        raise SaltInvocationError('Actions supported are "add" and "delete", not "{0}".'.format(action))
+    if action == 'add' and not sel_type:
+        raise SaltInvocationError('SELinux Type is required to add a policy')
+    (protocol, port) = _parse_protocol_port(name, protocol, port)
+    cmd = 'semanage port --{0} --proto {1}'.format(action, protocol)
+    if sel_type:
+        cmd += ' --type {0}'.format(sel_type)
+    if sel_range:
+        cmd += ' --range {0}'.format(sel_range)
+    cmd += ' {0}'.format(port)
+    return __salt__['cmd.run_all'](cmd)
